limitedLiability <- TRUE
allowInvest <- TRUE

p0 <- 100
k <- 80
r <- 0.05

I <- 600
D <- 30

sigma <- 0.10

T <- 10
deltat <- 1/100

# Anzahl der Zeitintervalle
tInts <- T/deltat

# Berechnung von u und d, damit die Standardabweichung von p gleich sigma ist und damit gilt E(p_(t+1)) = p_t
u <- exp(sigma*sqrt(deltat))
d <- 1/u
pi <- (1-d)/(u-d)

#nötig, damit Matrix-Indices OK sind
pOffset <- tInts+1
tOffset <- 1

# allokiere den Preis-Baum für das Problem x=5 (nach der Investition)
x <- 5
V5 <- matrix(NA,nrow=2*tInts+1,ncol=(tInts+1))

# am Ende ist die Maschine wertlos
V5[,tOffset+tInts] <- 0

# bereite den Plot vor
par(mfrow=c(2,1))
plot(c(0,T),c(k,k),t="l", ylim=c(0,150),ylab="p",xlab="t",mar=c(1,1,1,1),main="V(p,x=5)")


# der Zähler t läuft von tInts-1 bis 0, daher muss in der Matrix ein tOffset von 1 addiert werden
for(t in (tInts-1):0){

  print(t*deltat)

  # welche p Werte sind beim aktuellen t erreichbar? 
  attainableUps <- seq(from=-t, to=t, by=2)
  shutDownAt <- 0
  
  for(ups in attainableUps){

    # ups ist ein Zähler, der die Anzahl der up-moves angibt, die nötig sind, um von p0 aus nach p zu gelangen
    # negative ups entsprechen down-moves, da d=1/u gilt.
    p <- p0*u^ups

    cashflow <- x * (p-k) * deltat
    # das ist der Wert, den ich (in Erwartung) erhalte, wenn ich die Maschine einfach weiter laufen lasse.
    continuationValue <- cashflow+ 1/(1+r)^deltat*(pi*V5[pOffset+ups+1,tOffset+t+1]+(1-pi)*V5[pOffset+ups-1,tOffset+t+1])
    # nun kommt die Entscheidung: Ich kann die Maschine entweder weiter laufen lassen, oder durch Bezahlung von D schließen
    # wenn also continuationValue < -D gilt, ist es besser zuzusperren, den höchsten Wert, bei dem zugesperrt wird, merke ich
    # mir in shutDownAt
    if(limitedLiability){
      if(continuationValue<=-D){
        V5[pOffset+ups,tOffset+t] <- -D
        shutDownAt <- p
      } else {
        V5[pOffset+ups,tOffset+t] <- continuationValue
      }
    } else {
      V5[pOffset+ups,tOffset+t] <- continuationValue
    }
  }

  # hier wird gezeichnet, blau = laufen lassen, rot = zusperren
  if((round(t/5)==t/5) || (t==(tInts-1))){

    minp <- p0*u^min(attainableUps)
    maxp <- p0*u^max(attainableUps)
    if(shutDownAt>minp){
      lines(c(t*deltat,t*deltat),c(minp,shutDownAt),col=2)
      lines(c(t*deltat,t*deltat),c(shutDownAt*u,maxp),col=4)
    } else {
      lines(c(t*deltat,t*deltat),c(minp,maxp),col=4)
    }
  }

  
}


# nun das gleiche für x = 1

# allokiere den Preis-Baum für das Problem x=1 (nach der Investition)
x <- 1
V1 <- matrix(NA,nrow=2*tInts+1,ncol=(tInts+1))

# am Ende ist die Maschine wertlos
V1[,tOffset+tInts] <- 0

# bereite den Plot vor
plot(c(0,T),c(k,k),t="l", ylim=c(0,150),ylab="p",xlab="t",mar=c(1,1,1,1),main="V(p,x=1)")


# das gleiche wie oben
for(t in (tInts-1):0){

  print(t*deltat)

  attainableUps <- seq(from=-t, to=t, by=2)
  shutDownAt <- 0
  investAt <- 0
  
  for(ups in attainableUps){

    p <- p0*u^ups

    cashflow <- x * (p-k) * deltat
    continuationValue <- cashflow+ 1/(1+r)^deltat*(pi*V1[pOffset+ups+1,tOffset+t+1]+(1-pi)*V1[pOffset+ups-1,tOffset+t+1])
    # Entscheidung: warten ("weite laufen lassen"), investieren oder zusperren!
    if(limitedLiability){
      if(continuationValue<=-D){
        V1[pOffset+ups,tOffset+t] <- -D
        shutDownAt <- p
      } else {
        V1[pOffset+ups,tOffset+t] <- continuationValue
      }
    } else {
      V1[pOffset+ups,tOffset+t] <- continuationValue
    }
    # es ist optimal zu investieren, wenn der Wert der Maschine mit x=5 den Wert der Maschine mit x=1 um mehr als I übersteigt,
    # denn dann werde ich I bezahlen und die Kapazität auf 5 erweitern
    if(allowInvest &&  V1[pOffset+ups,tOffset+t] < (V5[pOffset+ups,tOffset+t]-I)){
       V1[pOffset+ups,tOffset+t] <-  V5[pOffset+ups,tOffset+t] -I
       # merke mir den niedrigsten Punkt, bei dem ich investiere in investAt
       if(investAt==0){investAt <- p}
    }
  }


  # hier wird gezeichnet, blau = laufen lassen, rot = zusperren, schwarz = investieren
  if((round(t/5)==t/5) || (t==(tInts-1))){

    minp <- p0*u^min(attainableUps)
    maxp <- p0*u^max(attainableUps)
    if(shutDownAt>minp){
      lines(c(t*deltat,t*deltat),c(minp,shutDownAt),col=2)
      lines(c(t*deltat,t*deltat),c(shutDownAt*u,maxp),col=4)
    } else {
      lines(c(t*deltat,t*deltat),c(minp,maxp),col=4)
    }
    if((investAt>0) && (investAt<maxp)){
      lines(c(t*deltat,t*deltat),c(investAt,maxp),col=1)
    }
    
  }

  
}
